#!/usr/bin/perl
# This is automatically generated by author/import-moose-test.pl.
# DO NOT EDIT THIS FILE. ANY CHANGES WILL BE LOST!!!
use lib "t/lib";
use MooseCompat;

use strict;
use warnings;

use Test::Builder::Tester;
use Test::More;

BEGIN {
  use_ok('Test::Mouse');
}

{
    package Foo;
    use Mouse;

    has 'foo', is => 'bare';
}

{
    package Bar;
    use Mouse;

    extends 'Foo';

    has 'bar', is => 'bare';
}


test_out('ok 1 - ... has_attribute_ok(Foo, foo) passes');

has_attribute_ok('Foo', 'foo', '... has_attribute_ok(Foo, foo) passes');

test_out ('not ok 2 - ... has_attribute_ok(Foo, bar) fails');
test_fail (+2);

has_attribute_ok('Foo', 'bar', '... has_attribute_ok(Foo, bar) fails');

test_out('ok 3 - ... has_attribute_ok(Bar, foo) passes');

has_attribute_ok('Bar', 'foo', '... has_attribute_ok(Bar, foo) passes');

test_out('ok 4 - ... has_attribute_ok(Bar, bar) passes');

has_attribute_ok('Bar', 'bar', '... has_attribute_ok(Bar, bar) passes');

test_test ('has_attribute_ok');

done_testing;
