
# This is automatically generated by author/import-moose-test.pl.
# DO NOT EDIT THIS FILE. ANY CHANGES WILL BE LOST!!!
use MooseCompat;
package Bar;
use Mouse;
use Mouse::Util::TypeConstraints;

type Baz => where { 1 };

subtype Bling => as Baz => where { 1 };

1;